/*  This file is part of gpkgdep.

    gpkgdep is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    gpkgdep is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with gpkgdep; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <stdio.h>
#include <stdlib.h>
#include "gtk/gtk.h"
#include "gdk/gdkkeysyms.h"
#include "gpkgdep.h"
#include "package_tree.h"
#include "req_pkg_tree.h"
#include "dep_pkg_tree.h"
#include "pkg_simul.h"

int packageLoadAborted = FALSE;
GtkWidget * window;
char * aboutText = "gpkgdep %d.%d:\nA graphical package dependency viewer for FreeBSD.\n\nWritten by Heiner Eichmann:\nh.eichmann@gmx.de";
char * helpText = "<span foreground=\"red\">Sorry for supplying the docmuentation in such a horrible window. "
    "All other gtk widgets were too hard to use. Can someone help me here?</span>\n\n"
    "<big><big><big>gpkgdep Documentation</big></big></big>\n\n"
    "<big><big>1. Introduction</big></big>\n"
    "gpkgdep is a package viewer for FreeBSD. It allows to visualize the dependencies of all installed packages "
    "in three different ways. The first shows the required, the second the dependent packages. The third "
    "offers a comfortable way to identify unnecessary packages interactively.\n\n"
    "<u><b>Note:</b> gpkgdep does only view things. It does not change anything on the packages!</u>\n\n"
    "Furthermore gpkgdep requires a consistent package database. Please install sysutils/portupgrade and run "
    "pkgdb -F before using gpkgdep. If the package database is inconsistent, a red warning line appears at "
    "the bottom of the main window!\n\n"
    "<big><big>2. Required Packages</big></big>\n"
    "The \"Required Packages\" tab shows all currently installed packages. If one requires another "
    "package, a small triangle appears before its name. By clicking on it, a list of the required packages "
    "appears, forming a small tree. This can be repeated recursively. Example: If you have KDE3 installed, "
    "clicking on the triangle reveals f.e. qt, because KDE3 requires qt.\n\n"
    "If a package or one of its childs requires a "
    "packages, which does not exist, it is displyed in red. Please close gpkgdep, run pkgdb -F and restart gpkgdep "
    "to fix this problem.\n\n"
    "<big><big>3. Dependent Packages</big></big>\n"
    "The \"Dependent Packages\" tab shows the same but in the opposite direction: all currently installed "
    "packages are shown as well. If another packages require one in the list, "
    "a small triangle appears before its name. By clicking on it, a list of these packages, "
    "which are requireing the starting one, is shown. Example: if you have KDE3 installed, qt will be in the list. "
    "Clicking on qt reveals KDE3, because KDE3 requires qt. Packages without childs can probably be deinstalled "
    "without making other packages unable to run.\n\n"
    "<big><big>4. Remove Package Simmulation</big></big>\n"
    "The \"Remove Package Simmulation\" tab again shows all currently installed packages. This time there is "
    "a check box in front of each package. Clicking on it marks the corresponding package as \"wanted\" package. "
    "Wanted packages are packages the user wanted to keep on his or hers computer. Marking one package \"wanted\" also "
    "marks all packages this one requires as \"wanted\"! Furthermore this required packages are disabled, so that "
    "the \"wanted\" check box can not be un-checked. Marking all \"wanted\" packages will automatically mark "
    "all packages these require. What remains \"unwanted\" are obviously good candidates for "
    "deinstallation.\n\n"
    "Please note the options at the bottom of the dialog:\n\n"
    "<big>Show all packages</big>\nshows, as the title implies, all packages.\n\n"
    "<big>Show wanted and unwanted packages</big>\nShows all packages which are \"unwanted\" (not checked) and \"wanted\" "
    "(checked). Packages required by the \"wanted\" are not shown.\n\n"
    "<big>Show unwanted packages</big>\n(The default) shows all packages which are \"unwanted\" (not checked)."
    "All \"wanted\" packages (checked by the user and required by them) are not shown.\n\n"
    "<big>Show wanted packages</big>\nOnly shows all packages which are \"wanted\" (checked) by the user. "
    "Packages which are required by them and which are \"unwanted\" are not shown.\n\n"
    "<big>Show wanted and dependent packages</big>\nShows all packages which are \"wanted\" (checked) either "
    "because they are checked by the user, or because they are required by them.\n\n"
    "<big>Show dependent packages</big>\nShows all which depend on the \"wanted\" packages.";

gint delete_event( GtkWidget *widget, GdkEvent  *event, gpointer   data )
{
    gtk_main_quit ();
    return FALSE;
}


gint help_event( GtkWidget *widget, GdkEvent  *event, gpointer   data )
{
    GtkWidget * dialog, * label, * srollWidget;

      label = gtk_label_new (NULL);
      //gtk_widget_set_size_request (label, 550, 100);
      gtk_label_set_markup (GTK_LABEL (label), helpText);
    dialog = gtk_dialog_new_with_buttons ("gpkgdep Help", GTK_WINDOW(window), GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_OK, GTK_RESPONSE_NONE, NULL);
    gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
    gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
    g_signal_connect_swapped (GTK_OBJECT (dialog), "response", G_CALLBACK (gtk_widget_destroy), GTK_OBJECT (dialog));
    srollWidget = gtk_scrolled_window_new (NULL, NULL);
    gtk_container_set_border_width (GTK_CONTAINER (srollWidget), 5);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (srollWidget), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
    gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (srollWidget), GTK_WIDGET (label));
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dialog)->vbox), srollWidget, TRUE, TRUE, 0);
//    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), srollWidget);
    gtk_widget_set_size_request (dialog, 400, 500);
    gtk_widget_show_all(dialog);
    return FALSE; // or FALSE??? not documented!!
}

gint about_event( GtkWidget *widget, GdkEvent  *event, gpointer   data )
{
    GtkWidget * dialog, * label;
    gchar * about = g_strdup_printf(aboutText, GPKGDEP_VERSION_MAJOR, GPKGDEP_VERSION_MINOR);

    dialog = gtk_dialog_new_with_buttons ("About gpkgdep", GTK_WINDOW(window), GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_OK, GTK_RESPONSE_NONE, NULL);
    label = gtk_label_new (about);
    gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_FILL);
    gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
    g_signal_connect_swapped (GTK_OBJECT (dialog), "response", G_CALLBACK (gtk_widget_destroy), GTK_OBJECT (dialog));
    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), label);
    gtk_widget_show_all(dialog);
    g_free(about);
    return FALSE; // or FALSE??? not documented!!
}

GtkWidget * createPkgWidget()
{
    GtkWidget * tabCtrl = gtk_notebook_new();
    gtk_notebook_append_page(GTK_NOTEBOOK(tabCtrl), createRequiredTreeWidget(), gtk_label_new("Required Packages"));
    gtk_notebook_append_page(GTK_NOTEBOOK(tabCtrl), createDependentTreeWidget(), gtk_label_new("Dependent Packages"));
    gtk_notebook_append_page(GTK_NOTEBOOK(tabCtrl), createPackageSimulatorWidget(), gtk_label_new("Remove Package Simulation"));
    gtk_widget_show(tabCtrl);
    return tabCtrl;
}

void cancelButtonClicked(GtkWidget *widget)
{
    packageLoadAborted = TRUE;
}

int loadPackagesCallback(int step, int maxStep, gpointer data)
{
    GtkWidget * progressBar = (GtkWidget *) data;
    double progress = (double)step/((double)maxStep);
    gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progressBar), progress);
    while (gtk_events_pending ())
        gtk_main_iteration ();
    return packageLoadAborted;
}

int main( int   argc,
          char *argv[])
{
    GtkWidget * box1;
    GtkWidget * menuBar;
    GtkWidget * menuItem;
    GtkWidget * menu;
    GtkWidget * warningLabel;
    GtkWidget * warningBox;
    GtkWidget * progressDialog;
    GtkWidget * progressBox;
    GtkWidget * progressBar;
    GtkAccelGroup *accelGroup;
    GdkColor color;
    char warningText[256];
    int invalidLinks;

    gtk_init (&argc, &argv);

    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    accelGroup = gtk_accel_group_new ();
    gtk_window_add_accel_group (GTK_WINDOW (window), accelGroup);

    g_signal_connect (G_OBJECT (window), "delete_event", G_CALLBACK (delete_event), NULL);

    box1 = gtk_vbox_new (FALSE, 0);


    warningLabel = gtk_label_new("");
    gtk_container_set_border_width (GTK_CONTAINER (window), 5);
    gtk_label_set_justify(GTK_LABEL(warningLabel), GTK_JUSTIFY_LEFT);

    menuBar = gtk_menu_bar_new();
    menu = gtk_menu_new();
    gtk_menu_set_accel_group (GTK_MENU (menu), accelGroup);
    menuItem = gtk_menu_item_new_with_mnemonic ("_File");
    gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuItem), menu);
    gtk_menu_bar_append (GTK_MENU_BAR (menuBar), menuItem);
    menuItem = gtk_menu_item_new_with_mnemonic ("_Quit");
    gtk_widget_add_accelerator (menuItem, "activate", accelGroup, GDK_F4, GDK_Alt_R, GTK_ACCEL_VISIBLE);
    g_signal_connect (G_OBJECT (menuItem), "activate", G_CALLBACK (delete_event), NULL);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuItem);
    menu = gtk_menu_new();
    menuItem = gtk_menu_item_new_with_mnemonic ("_Help");
    gtk_menu_item_set_right_justified (GTK_MENU_ITEM (menuItem), TRUE);
    gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuItem), menu);
    gtk_menu_bar_append (GTK_MENU_BAR (menuBar), menuItem);
    menuItem = gtk_menu_item_new_with_mnemonic ("_About gpkgdep");
    g_signal_connect (G_OBJECT (menuItem), "activate", G_CALLBACK (about_event), NULL);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuItem);
    menuItem = gtk_menu_item_new_with_mnemonic ("_Help");
    gtk_widget_add_accelerator (menuItem, "activate", accelGroup, GDK_F1, 0, GTK_ACCEL_VISIBLE);
    g_signal_connect (G_OBJECT (menuItem), "activate", G_CALLBACK (help_event), NULL);

    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuItem);

    gtk_box_pack_start (GTK_BOX (box1), menuBar, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (box1), createPkgWidget(), TRUE, TRUE, 0);
    warningBox = gtk_vbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (warningBox), warningLabel, TRUE, TRUE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (warningBox), 5);
    gtk_box_pack_start (GTK_BOX (box1), warningBox, FALSE, FALSE, 0);

    gtk_container_add (GTK_CONTAINER (window), box1);
    gtk_widget_set_sensitive(window, FALSE);

    progressDialog = gtk_dialog_new_with_buttons ("Loading Package Dependencies", GTK_WINDOW(window), GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_CANCEL, GTK_RESPONSE_NONE, NULL);
    gtk_widget_set_size_request (progressDialog, 300, 100);
    progressBar = gtk_progress_bar_new();
    progressBox = gtk_vbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (progressBox), progressBar, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (progressBox), gtk_label_new ("Please wait..."), FALSE, FALSE, 0);
    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(progressDialog)->vbox), progressBox);
    g_signal_connect_swapped (GTK_OBJECT (progressDialog), "response", G_CALLBACK (cancelButtonClicked), GTK_OBJECT (progressDialog));

    gtk_window_set_default_size(GTK_WINDOW(window), 600, 500);
    gtk_widget_show_all (window);
    gtk_widget_show_all (progressDialog);

    if (!loadPackageTree(loadPackagesCallback, progressBar))
        return -1;
    invalidLinks = numberOfInvalidLinks();
    if (invalidLinks > 0)
    {
        if (invalidLinks == 1)
            sprintf(warningText, "ATTENTION: one invalid link. Please run pkgdb -F to fix!", invalidLinks);
        else
            sprintf(warningText, "ATTENTION: %d invalid links. Please run pkgdb -F to fix!", invalidLinks);
        gdk_color_parse ("red", &color);
        gtk_widget_modify_fg (warningLabel, GTK_STATE_NORMAL, &color);
        gtk_label_set_text(GTK_LABEL(warningLabel), warningText);
    }
    fillRequiredTree();
    fillDependentTree();
    fillPackageSimulatorTree();
    gtk_widget_destroy(progressDialog);
    gtk_widget_set_sensitive(window, TRUE);

    gtk_main ();

    deletePackageTree();
    return 0;
}

